'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

require('../../../src/stylus/components/_autocompletes.styl');

var _VSelect = require('../VSelect/VSelect');

var _VSelect2 = _interopRequireDefault(_VSelect);

var _VTextField = require('../VTextField/VTextField');

var _VTextField2 = _interopRequireDefault(_VTextField);

var _helpers = require('../../util/helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Styles
exports.default = {
  name: 'v-autocomplete',

  extends: _VSelect2.default,

  data: function data(vm) {
    return {
      attrsInput: null,
      lazySearch: vm.searchInput,
      lazyValue: vm.value != null ? vm.value : vm.multi || vm.tags ? [] : undefined
    };
  },

  props: {
    browserAutocomplete: {
      type: String,
      default: 'off'
    },
    combobox: Boolean,
    delimiters: Array,
    filter: {
      type: Function,
      default: function _default(item, queryText, itemText) {
        var hasValue = function hasValue(val) {
          return val != null ? val : '';
        };

        var text = hasValue(itemText);
        var query = hasValue(queryText);

        return text.toString().toLowerCase().indexOf(query.toString().toLowerCase()) > -1;
      }
    },
    noFilter: Boolean,
    offsetY: {
      type: Boolean,
      default: true
    },
    searchInput: {
      default: null
    },
    tags: Boolean,
    transition: {
      type: [Boolean, String],
      default: false
    }
  },

  computed: {
    classes: function classes() {
      return Object.assign({}, _VSelect2.default.computed.classes.call(this), {
        'v-autocomplete': true,
        'v-autocomplete--is-selecting-index': this.selectedIndex > -1
      });
    },
    computedItems: function computedItems() {
      return this.filteredItems;
    },

    /**
     * The range of the current input text
     *
     * @return {Number}
     */
    currentRange: function currentRange() {
      if (this.selectedItem == null) return 0;

      return this.getText(this.selectedItem).toString().length;
    },
    filteredItems: function filteredItems() {
      var _this = this;

      var items = this.filterDuplicates(this.cachedItems.concat(this.items));

      if (!this.isSearching || this.noFilter) return items;

      return items.filter(function (i) {
        return _this.filter(i, _this.internalSearch, _this.getText(i));
      });
    },
    hasSlot: function hasSlot() {
      return _VSelect2.default.computed.hasSlot.call(this) || this.tags;
    },

    internalSearch: {
      get: function get() {
        return this.lazySearch;
      },
      set: function set(val) {
        this.lazySearch = val;

        this.$emit('update:searchInput', val);
      }
    },
    isAnyValueAllowed: function isAnyValueAllowed() {
      return this.combobox || this.tags;
    },
    isDirty: function isDirty() {
      return this.searchIsDirty || this.selectedItems.length > 0;
    },
    isMulti: function isMulti() {
      return this.tags || _VSelect2.default.computed.isMulti.call(this);
    },
    isSearching: function isSearching() {
      if (this.isMulti) return this.searchIsDirty;

      return this.internalSearch !== this.getText(this.internalValue);
    },
    menuCanShow: function menuCanShow() {
      if (!this.isFocused) return false;

      var filtered = this.filteredItems.length > 0;

      if (this.isAnyValueAllowed) {
        return filtered;
      }

      return !this.hideNoData || filtered;
    },
    menuProps: function menuProps() {
      return Object.assign(_VSelect2.default.computed.menuProps.call(this), {
        contentClass: 'v-autocomplete__content',
        value: this.menuCanShow && this.isMenuActive
      });
    },
    searchIsDirty: function searchIsDirty() {
      return this.internalSearch != null && this.internalSearch !== '';
    },
    selectedItem: function selectedItem() {
      var _this2 = this;

      if (this.isMulti) return null;

      return this.selectedItems.find(function (i) {
        return _this2.valueComparator(_this2.getValue(i), _this2.getValue(_this2.internalValue));
      });
    }
  },

  watch: {
    filteredItems: function filteredItems() {
      this.setMenuIndex(-1);
    },
    isFocused: function isFocused(val) {
      if (val) {
        this.$refs.input && this.$refs.input.select();
      }
    },
    isMenuActive: function isMenuActive(val) {
      if (val || !this.hasSlot) return;

      this.lazySearch = null;
    },
    searchInput: function searchInput(val) {
      this.lazySearch = val;
    },
    internalSearch: function internalSearch(val) {
      if (val && this.tags && this.delimiters) {
        var delimiter = this.delimiters.find(function (d) {
          return val.endsWith(d);
        });
        if (delimiter == null) return;

        this.internalSearch = val.slice(0, val.length - delimiter.length);
        this.updateTags();
      }
    }
  },

  created: function created() {
    this.setSearch();
  },


  methods: {
    activateMenu: function activateMenu() {
      if (this.menuCanShow) {
        this.isMenuActive = true;
      }
    },
    changeSelectedIndex: function changeSelectedIndex(keyCode) {
      // Do not allow changing of selectedIndex
      // when search is dirty
      if (this.searchIsDirty) return;

      if (![_helpers.keyCodes.backspace, _helpers.keyCodes.left, _helpers.keyCodes.right, _helpers.keyCodes.delete].includes(keyCode)) return;

      var indexes = this.selectedItems.length - 1;

      if (keyCode === _helpers.keyCodes.left) {
        this.selectedIndex = this.selectedIndex === -1 ? indexes : this.selectedIndex - 1;
      } else if (keyCode === _helpers.keyCodes.right) {
        this.selectedIndex = this.selectedIndex >= indexes ? -1 : this.selectedIndex + 1;
      } else if (this.selectedIndex === -1) {
        this.selectedIndex = indexes;
        return;
      }

      var currentItem = this.selectedItems[this.selectedIndex];

      if ([_helpers.keyCodes.backspace, _helpers.keyCodes.delete].includes(keyCode) && !this.getDisabled(currentItem)) {
        var newIndex = this.selectedIndex === indexes ? this.selectedIndex - 1 : this.selectedItems[this.selectedIndex + 1] ? this.selectedIndex : -1;

        if (newIndex === -1) {
          this.internalValue = this.isMulti ? [] : undefined;
        } else {
          this.selectItem(currentItem);
        }

        this.selectedIndex = newIndex;
      }
    },
    genInput: function genInput() {
      var input = _VTextField2.default.methods.genInput.call(this);

      input.data.attrs.role = 'combobox';
      input.data.domProps.value = this.internalSearch;

      return input;
    },
    genList: function genList() {
      var list = _VSelect2.default.methods.genList.call(this);

      list.componentOptions.propsData.items = this.filteredItems;
      list.componentOptions.propsData.noFilter = this.noFilter || !this.isSearching || !this.filteredItems.length;
      list.componentOptions.propsData.searchInput = this.internalSearch;

      return list;
    },
    genSelections: function genSelections() {
      return this.hasSlot || this.isMulti ? _VSelect2.default.methods.genSelections.call(this) : [];
    },
    onBlur: function onBlur(e) {
      this.updateSelf();
      _VSelect2.default.methods.onBlur.call(this, e);
    },
    onClick: function onClick() {
      if (this.isDisabled) return;

      this.selectedIndex > -1 ? this.selectedIndex = -1 : this.onFocus();

      this.activateMenu();
    },
    onEnterDown: function onEnterDown() {
      this.onBlur();
    },
    onEscDown: function onEscDown(e) {
      e.preventDefault();
      this.isMenuActive = false;
    },
    onInput: function onInput(e) {
      // If typing and menu is not currently active
      if (e.target.value) {
        this.activateMenu();
        if (!this.isAnyValueAllowed) this.setMenuIndex(0);
      }

      this.mask && this.resetSelections(e.target);
      this.internalSearch = e.target.value;
      this.badInput = e.target.validity && e.target.validity.badInput;
    },
    onKeyDown: function onKeyDown(e) {
      var keyCode = e.keyCode;

      // If enter, space, up, or down is pressed, open menu
      if (!this.isMenuActive && [_helpers.keyCodes.enter, _helpers.keyCodes.space, _helpers.keyCodes.up, _helpers.keyCodes.down].includes(keyCode)) this.activateMenu();

      // This should do something different
      if (e.keyCode === _helpers.keyCodes.enter) return this.onEnterDown();

      // If escape deactivate the menu
      if (keyCode === _helpers.keyCodes.esc) return this.onEscDown(e);

      // If tab - select item or close menu
      if (keyCode === _helpers.keyCodes.tab) return this.onTabDown(e);

      if (this.$refs.input.selectionStart === 0) this.updateSelf();
      if (!this.hideSelections) this.changeSelectedIndex(keyCode);
    },
    onTabDown: function onTabDown(e) {
      var _this3 = this;

      var menuIndex = this.getMenuIndex();

      // When adding tags, if searching and
      // there is not a filtered options,
      // add the value to the tags list
      if (this.tags && this.internalSearch && menuIndex === -1) {
        e.preventDefault();
        e.stopPropagation();

        return this.updateTags();
        // An item that is selected by
        // menu-index should toggled
      } else if (this.isMenuActive && menuIndex > -1) {
        // Reset the list index if searching
        this.internalSearch && this.$nextTick(function () {
          return setTimeout(function () {
            return _this3.setMenuIndex(-1);
          }, 0);
        });

        e.preventDefault();
        this.selectListTile(menuIndex);
      } else {
        // If we make it here,
        // the user has no selected indexes
        // and is probably tabbing out
        this.isFocused = false;
      }

      this.updateSelf();
    },
    selectItem: function selectItem(item) {
      _VSelect2.default.methods.selectItem.call(this, item);

      this.setSearch();
    },
    selectListTile: function selectListTile(index) {
      if (!this.$refs.menu.tiles[index]) return;

      this.$refs.menu.tiles[index].click();
    },
    setSelectedItems: function setSelectedItems() {
      if (this.internalValue == null) {
        this.selectedItems = [];
      } else if (this.tags) {
        this.selectedItems = this.internalValue;
      } else if (this.combobox) {
        this.selectedItems = [this.internalValue];
      } else {
        _VSelect2.default.methods.setSelectedItems.call(this);
      }
    },
    setMenuIndex: function setMenuIndex(index) {
      this.$refs.menu && (this.$refs.menu.listIndex = index);
    },
    setSearch: function setSearch() {
      if (this.hasSlot || this.isMulti) return;

      this.lazySearch = this.getText(this.internalValue);
    },
    setValue: function setValue() {
      this.internalValue = this.internalSearch;
      this.$emit('change', this.internalSearch);
    },
    updateAutocomplete: function updateAutocomplete() {
      if (!this.searchIsDirty && !this.internalValue) return;

      if (this.isMulti) return this.internalSearch = null;

      if (!this.valueComparator(this.internalSearch, this.getValue(this.internalValue))) {
        this.internalSearch = this.initialValue;
      }
    },
    updateCombobox: function updateCombobox() {
      if (this.internalSearch !== this.internalValue) this.setValue();else this.updateAutocomplete();
    },
    updateSelf: function updateSelf() {
      if (this.tags) this.updateTags();else if (this.combobox) this.updateCombobox();else this.updateAutocomplete();
    },

    // Maybe change to onBlur?
    updateTags: function updateTags() {
      var menuIndex = this.getMenuIndex();

      // If the user is not searching
      // and no menu item is selected
      // do nothing
      if (menuIndex < 0 && !this.searchIsDirty) return;

      var index = this.selectedItems.indexOf(this.internalSearch);
      // If it already exists, do nothing
      // this might need to change to bring
      // the duplicated item to the last entered
      if (index > -1) {
        this.internalValue.splice(index, 1);
      }

      // If menu index is greater than 1
      // the selection is handled elsewhere
      // TODO: find out where
      if (menuIndex > -1) return this.internalSearch = null;

      this.selectItem(this.internalSearch);
      this.internalSearch = null;
    }
  }
};

// Utils


// Extensions