'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _registrable = require('../../mixins/registrable');

exports.default = {
  name: 'v-form',

  inheritAttrs: false,

  mixins: [(0, _registrable.provide)('form')],

  data: function data() {
    return {
      inputs: [],
      errorBag: {}
    };
  },


  props: {
    value: Boolean,
    lazyValidation: Boolean
  },

  watch: {
    errorBag: {
      handler: function handler() {
        var errors = Object.values(this.errorBag).includes(true);
        this.$emit('input', !errors);
      },

      deep: true,
      immediate: true
    }
  },

  methods: {
    watchInput: function watchInput(input) {
      var _this = this;

      var valid = input.$watch('valid', function (val) {
        if (!input.shouldValidate && _this.lazyValidation) return;
        _this.$set(_this.errorBag, input._uid, !val);
      }, { immediate: true });

      var shouldValidate = input.$watch('shouldValidate', function (val) {
        if (val) _this.$set(_this.errorBag, input._uid, !input.valid);
      }, { immediate: true });

      return { valid: valid, shouldValidate: shouldValidate };
    },
    validate: function validate() {
      var errors = this.inputs.filter(function (input) {
        return !input.validate(true);
      }).length;
      return !errors;
    },
    reset: function reset() {
      for (var i = this.inputs.length; i--;) {
        this.inputs[i].reset();
      }
      if (this.lazyValidation) this.errorBag = {};
    },
    register: function register(input) {
      var unwatch = this.watchInput(input);
      this.inputs.push({
        uid: input._uid,
        validate: input.validate,
        reset: input.reset,
        unwatch: unwatch
      });
    },
    unregister: function unregister(input) {
      var found = this.inputs.find(function (i) {
        return i.uid === input._uid;
      });

      if (!found) return;

      found.unwatch.valid();
      found.unwatch.shouldValidate();
      this.inputs = this.inputs.filter(function (i) {
        return i.uid !== found.uid;
      });
      this.$delete(this.errorBag, found.uid);
    }
  },

  render: function render(h) {
    var _this2 = this;

    return h('form', {
      attrs: Object.assign({
        novalidate: true
      }, this.$attrs),
      on: {
        submit: function submit(e) {
          return _this2.$emit('submit', e);
        }
      }
    }, this.$slots.default);
  }
};