
import { provide as RegistrableProvide } from '../../mixins/registrable'

export default {
  name: 'v-form',

  inheritAttrs: false,

  mixins: [RegistrableProvide('form')],

  data () {
    return {
      inputs: [],
      errorBag: {}
    }
  },

  props: {
    value: Boolean,
    lazyValidation: Boolean
  },

  watch: {
    errorBag: {
      handler () {
        const errors = Object.values(this.errorBag).includes(true)
        this.$emit('input', !errors)
      },
      deep: true,
      immediate: true
    }
  },

  methods: {
    watchInput (input) {
      const valid = input.$watch('valid', val => {
        if (!input.shouldValidate && this.lazyValidation) return
        this.$set(this.errorBag, input._uid, !val)
      }, { immediate: true })

      const shouldValidate = input.$watch('shouldValidate', val => {
        if (val) this.$set(this.errorBag, input._uid, !input.valid)
      }, { immediate: true })

      return { valid, shouldValidate }
    },
    validate () {
      const errors = this.inputs.filter(input => !input.validate(true)).length
      return !errors
    },
    reset () {
      for (let i = this.inputs.length; i--;) {
        this.inputs[i].reset()
      }
      if (this.lazyValidation) this.errorBag = {}
    },
    register (input) {
      const unwatch = this.watchInput(input)
      this.inputs.push({
        uid: input._uid,
        validate: input.validate,
        reset: input.reset,
        unwatch
      })
    },
    unregister (input) {
      const found = this.inputs.find(i => i.uid === input._uid)

      if (!found) return

      found.unwatch.valid()
      found.unwatch.shouldValidate()
      this.inputs = this.inputs.filter(i => i.uid !== found.uid)
      this.$delete(this.errorBag, found.uid)
    }
  },

  render (h) {
    return h('form', {
      attrs: Object.assign({
        novalidate: true
      }, this.$attrs),
      on: {
        submit: e => this.$emit('submit', e)
      }
    }, this.$slots.default)
  }
}
